﻿IM = InterruptManager
IM.rotation = {}

function IM:GetVersion() return 148 end
local interruptSpells = IM.GetInterruptSpells()

function IM:SavePosition()
    InterruptManagerAnchor:StopMovingOrSizing()
    local a = IMDB.anchorPoint
    a.point, a.region, a.relativePoint, a.x, a.y = InterruptManagerAnchor:GetPoint()
end

--[[    IMDB = {
                [1] = Anchor:GetPoint()
                [2] = Anchor:GetSize()
                [3] = Anchor:GetMovable()
                [4] = Anchor:GetAlpha()
                [5] = Icon:GetPoint(x,y)
                [6] = Icon:GetSize()
                [7] = Icon:GetAlpha()
                [8] = Icon.text:GetFontSize(),Icon.text:GetFontAlpha()
                [9] = StatusBar:GetSize()
                [10]= StatusBar:GetPoint(x,y)
                [11]= StatusBar:GetAlpha()
                [12]= StatusBar.text:GetSize(),StatusBar.text:GetAlpha()
                [13]= SoloMode
                [14]= Version
                [15]= Announce
                [16]= PUGmode
                [17]= TargetWarn
                [18]= FocusWarn
                [19]= AnnounceAfterPUGmodeOff
                }
]]

function IM:InitializeSavedVariables()
    if (not IMDB) then
        IMDB = {}
    end
    IMDB.anchorPoint = IMDB.anchorPoint or {["point"] = "CENTER", ["region"] = nil, ["relativePoint"] = "CENTER", ["x"] = 0, ["y"] = -300}
    IMDB.anchorSize = IMDB.anchorSize or {["width"] = 200, ["height"] = 100}
    IMDB.anchorMovable = (IMDB.anchorMovable ~= false and IMDB.anchorMovable ~= true) or (IMDB.anchorMovable == true)
    IMDB.anchorAlpha = IMDB.anchorAlpha or 0.2
    
    IMDB.statusBarSize = IMDB.statusBarSize or {["width"] = 180, ["height"] = 20}
    IMDB.statusBarAlpha = IMDB.statusBarAlpha or 1
    IMDB.statusBarTextSize = IMDB.statusBarTextSize or 12
    IMDB.statusBarTextAlpha = IMDB.statusBarTextAlpha or 1
    
    IMDB.iconSize = IMDB.statusBarSize.height
    IMDB.iconAlpha = IMDB.iconAlpha or 0.4
    IMDB.iconTextSize = IMDB.iconTextSize or 12
    IMDB.iconTextAlpha = IMDB.iconTextAlpha or 1
    
    IMDB.announce = (IMDB.announce ~= false and IMDB.announce ~= true) or (IMDB.announce == true)
    IMDB.targetWarn = (IMDB.targetWarn ~= false and IMDB.targetWarn ~= true) or (IMDB.targetWarn == true)
    IMDB.announceChannel = IMDB.announceChannel or "SAY"
    IMDB.pugModeChannel = IMDB.pugModeChannel or "SAY"
    IMDB.numInterrupters = IMDB.numInterrupters or 0
    
    IMDB.version = IM.GetVersion()
end

function IM:InitializeInterrupterTable()
    IMDB.interrupters = {}
    
    for i = 1,5 do
        IMDB.interrupters[i] = {}
        IMDB.interrupters[i].active = false
        IMDB.interrupters[i].spells = {}
        IMDB.interrupters[i].name = ""
        IMDB.interrupters[i].ready = false
        IMDB.interrupters[i].next = false
        IMDB.interrupters[i].pos = i
        IMDB.interrupters[i].readyTime = 0
        IMDB.interrupters[i].cooldown = 0
        
        IM.rotation[i] = ""
    end
end

function IM:VariablesLoaded()
    if (IMDB) then
        IM.previousVersion = IMDB.version or 0
    else
        IM.previousVersion = 0
    end
    
    if (not IMDB or not IMDB.version or IMDB.version < 143) then
        IMDB = {}
        IM:InitializeInterrupterTable()
    end
    
    if (not IMDB.interrupters) then
        IM:InitializeInterrupterTable()
    end
    
    IM:InitializeSavedVariables()
    
    -- Create the main frame
    local f = CreateFrame("Frame", "InterruptManagerAnchor", UIParent)
    f:SetSize(IMDB.anchorSize.width, IMDB.anchorSize.height)
    f:SetPoint(IMDB.anchorPoint.point, IMDB.anchorPoint.region, IMDB.anchorPoint.relativePoint, IMDB.anchorPoint.x, IMDB.anchorPoint.y)
    if (IMDB.anchorMovable) then
        f:SetMovable(true)
        f:SetScript("OnMouseDown", function() f:StartMoving() end)
        f:SetScript("OnMouseUp", IM.SavePosition)
    end
    local t = f:CreateTexture()
    t:SetTexture(0, 0, 0, IMDB.anchorAlpha)
    t:SetAllPoints(f)
    
    -- Create each individual bar
    for i = 1,5 do
        local f = CreateFrame("Frame", "InterruptManagerIcon" .. i, InterruptManagerAnchor)
        f:SetSize(IMDB.iconSize, IMDB.iconSize)
        f:SetPoint("TOPLEFT", "InterruptManagerAnchor", "TOPLEFT", 0, -(i-1) * IMDB.iconSize)
        
        local t = f:CreateTexture()
        t:SetAllPoints(f)
        t:SetTexture(0, 0, 0, IMDB.iconAlpha)
        
        t = f:CreateFontString()
        t:SetPoint("CENTER", "InterruptManagerIcon" .. i, "CENTER")
        t:SetFont("Fonts\\FRIZQT__.TTF", IMDB.iconTextSize)
        t:SetText(i)
        t:SetTextColor(1, 1, 1, IMDB.iconTextAlpha)
    
        f = CreateFrame("StatusBar", "InterruptManagerStatusBar" .. i, _G["InterruptManagerIcon" .. i])
        f:SetSize(IMDB.statusBarSize.width, IMDB.statusBarSize.height)
        f:SetPoint("LEFT", "InterruptManagerIcon" .. i, "RIGHT")
        f:SetOrientation("HORIZONTAL")
        f:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
        f:SetStatusBarColor(0, 1, 0, IMDB.statusBarAlpha)
        f:SetFrameLevel(3)
        f:SetMinMaxValues(0, 1)
        f:SetValue(0)
        
        f.text = f:CreateFontString("InterruptManagerStatusBarText" .. i)
        f.text:SetPoint("LEFT", "InterruptManagerStatusBar" .. i, "LEFT", 5, 0)
        f.text:SetFont("Fonts\\FRIZQT__.TTF", IMDB.statusBarTextSize)
        f.text:SetTextColor(1, 1, 1, IMDB.statusBarTextAlpha)
        f.text:SetText(IMDB.interrupters[i].name)
        
        f.cooldownText = f:CreateFontString("InterruptManagerStatusBarCooldownText" .. i)
        f.cooldownText:SetSize(IMDB.statusBarTextSize*3, IMDB.statusBarTextSize)
        f.cooldownText:SetJustifyH("LEFT")
        f.cooldownText:SetPoint("RIGHT", "InterruptManagerStatusBar" .. i, "RIGHT", 3, 0)
        f.cooldownText:SetFont("Fonts\\FRIZQT__.TTF", IMDB.statusBarTextSize, "OUTLINE")
        f.cooldownText:SetTextColor(1, 1, 1, IMDB.statusBarTextAlpha)
        
        --local g = CreateFrame("StatusBar", "InterruptManagerSecondaryStatusBar"..i)
        --g:SetSize(IMDB.statusBarSize[1],IMDB.statusBarSize[2])
        --g:SetPoint("CENTER",f,"CENTER")
        --g:SetParent(f)
        --g:SetOrientation("HORIZONTAL")
        --g:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
        --g:SetStatusBarColor(1,0,0,IMDB.statusBarAlpha/2)
        --g:SetFrameLevel(2)
        --g:SetMinMaxValues(0,1)
        --g:SetValue(0)
    end
    
    -- Create the mid-screen warning text frame
    local c = CreateFrame("MessageFrame","InterruptManagerText")
    c:SetWidth(300)
    c:SetHeight(50)
    c:SetPoint("CENTER", UIParent, "CENTER", 0, 80)
    c:SetFont("Fonts\\FRIZQT__.TTF", 40)
    c:SetFadeDuration(1)
    c:SetTimeVisible(2)
    IM:OnLogin()
    f:SetScript("OnUpdate",IM.OnUpdate)
    
    RegisterAddonMessagePrefix("InterruptManager")
end

function IM:OnUpdate()
    for k,v in pairs(IMDB.interrupters) do
        if (v.active and not v.ready) then
            local bar = _G["InterruptManagerStatusBar" .. v.pos]
            
            -- Interrupt spell ready/not ready
            if (v.readyTime - GetTime() <= 0) then
                v.ready = true
                bar.cooldownText:SetText()
                return
            end
            
            -- Update StatusBar values
            bar:SetValue(v.readyTime - GetTime())
            
            -- Update text to reflect cooldown
            bar.cooldownText:SetText(round(v.readyTime - GetTime(),1))
        end
    end
end

function IM:OnEvent(event, ...)
    if event == "COMBAT_LOG_EVENT_UNFILTERED" then
        IM:CombatLogEvent(...)
    elseif event == "VARIABLES_LOADED" then
        IM:VariablesLoaded()
    elseif event == "PARTY_MEMBERS_CHANGED" then
    elseif event == "CHAT_MSG_ADDON" and ... == "InterruptManager" then
        IM:AddonMessageReceived(...)
    elseif event == "CHAT_MSG_SYSTEM" then
        IM:ChatMsgSystem(...)
    elseif event == "PLAYER_REGEN_DISABLED" then
        IM:EnterCombat()
    elseif event == "PLAYER_REGEN_ENABLED" then
        IM:LeaveCombat()
    elseif (event == "UNIT_SPELLCAST_START") then
        IM:UnitSpellcastStart(...)
    end
end

function IM:EnterCombat()
    
end

function IM:LeaveCombat()
    
end

f = CreateFrame("Frame","InterruptManager222")
f:SetScript("OnEvent",IM.OnEvent)
f:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
f:RegisterEvent("VARIABLES_LOADED")
f:RegisterEvent("PARTY_MEMBERS_CHANGED")
f:RegisterEvent("CHAT_MSG_ADDON")
f:RegisterEvent("CHAT_MSG_SYSTEM")
f:RegisterEvent("PLAYER_REGEN_DISABLED")
f:RegisterEvent("PLAYER_REGEN_ENABLED")
f:RegisterEvent("UNIT_SPELLCAST_START")

function IM:PlayerAtPosition(position)
    -- This function returns the full name of the player at the specified position in the rotation, including server name regardless of your server
    for k,v in pairs(IMDB.interrupters) do
        if (v.pos == position) then
            local name = v.name
            
            if (not strfind(v.name, "-")) then
                name = v.name .. "-" .. GetRealmName()
            end
            
            return name
        end
    end
    error("This function is supposed to always return a value.")
end

function IM:SortTable()
    -- 1: Place the readyTimes of active interrupters into a table
    -- 2: Sort the table in ascending order
    -- 3: Use the sorted values as keys in a new table, and assign ascending values to them
    -- 4: Set position values to the interrupters with matching readyTimes.
    local containerTable = {}
    local sortTable = {}
    
    for k,v in pairs(IMDB.interrupters) do
        if (v.active) then
            tinsert(sortTable, v.readyTime)
        end
    end
    
    sort(sortTable)
    for k,v in pairs(sortTable) do
        containerTable[v] = k
    end
    
    for k,v in pairs(IMDB.interrupters) do
        v.pos = containerTable[v.readyTime]
    end
end

function IM:UpdateInterruptRotation()
    local db = IMDB.interrupters
    IM:SortTable()
    
    -- Update the UI to reflect the updated rotation
    IM:UpdateStatusBarVisibility()
end

function IM:ResetInactiveStatusBars()
    for i = IMDB.numInterrupters+1,5 do
        local bar = _G["InterruptManagerStatusBar" .. i]
        _G["InterruptManagerIcon" .. i]:Hide()
        bar.text:SetText("")
        bar.cooldownText:SetText("")
        bar:SetValue(0)
    end
end

function IM:ActivateStatusBars()
    for i = 1,5 do
        local a = IMDB.interrupters[i]
        if (a.active) then
            local bar = _G["InterruptManagerStatusBar" .. a.pos]
            _G["InterruptManagerIcon" .. a.pos]:Show()
            
            bar.text:SetText(a.name)
            bar:SetMinMaxValues(0, a.cooldown)
        end
    end
end

function IM:UpdateStatusBarVisibility()
    -- Sets the name of each interrupter to the appropriate status bar, according to their position in the rotation
    -- Sets minMax values of the StatusBars
    -- Shows or hides StatusBars depending on their activity
    -- Scales the anchor to cover no more than the active StatusBars
    if (IMDB.numInterrupters > 0) then
        InterruptManagerAnchor:Show()
        InterruptManagerAnchor:SetSize(IMDB.anchorSize.width, IMDB.anchorSize.height * 0.2 * IMDB.numInterrupters)
    else
        InterruptManagerAnchor:Hide()
    end
    
    IM:ActivateStatusBars()
    IM:ResetInactiveStatusBars()
end

function IM:PugModeInterruptHandler()
    local db = IMDB.interrupters
    
    if (IMDB.pugMode) then
        local c = IMDB.pugModeChannel
        
        if (c == "WHISPER") then
            SendChatMessage("InterruptManager: You are interrupting next", "WHISPER", nil, IM:PlayerAtPosition(1))
        else
        
            for i = IMDB.numInterrupters,1,-1 do
                
                local channel
                if (c == "RAID_WARNING" and IsInRaid(LE_PARTY_CATEGORY_HOME) and (UnitIsGroupLeader("player") or UnitIsGroupAssistant("player"))) then
                    channel = "RAID_WARNING"
                elseif ((c == "RAID" or c == "RAID_WARNING") and IsInRaid(LE_PARTY_CATEGORY_HOME)) then
                    channel = "RAID"
                elseif ((c == "RAID" or c == "PARTY") and IsInGroup(LE_PARTY_CATEGORY_HOME)) then
                    channel = "PARTY"
                elseif (c == "INSTANCE_CHAT" and (IsInGroup(LE_PARTY_CATEGORY_INSTANCE) or IsInRaid(LE_PARTY_CATEGORY_INSTANCE))) then
                    channel = "INSTANCE_CHAT"
                elseif (c == "YELL") then
                    channel = "YELL"
                else
                    channel = "SAY"
                end
                
                if (i == 1) then
                    SendChatMessage("Interrupting next: " .. IM:PlayerAtPosition(i), channel)
                end
            end
            
        end
    end
end

function IM:HideOutgoingWhisper(self, event, msg)
    if (msg == "InterruptManager: You are interrupting next") then
        return true
    else
        return false
    end
end
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", IM.HideOutgoingWhisper)

function IM:InterruptUsed(sourceName, spellId)
    local db = IMDB.interrupters
        
    for i = 1,5 do
        if (sourceName == db[i].name) then
            for k,v in pairs(interruptSpells.spellId) do
                if (v == spellId) then
                    db[i].cooldown = interruptSpells.cooldown[k]
                    db[i].readyTime = GetTime() + interruptSpells.cooldown[k]
                    db[i].ready = false
                    IM:UpdateInterruptRotation()
                    IM:PugModeInterruptHandler()
                end
            end
        end
    end
end

-- Function for announcing my own interrupt, as well as interrupt rotation in pug-mode
function IM:AnnounceMyInterrupt(spellName)
    local c = IMDB.announceChannel
    
    if (c == "RAID_WARNING" and IsInRaid(LE_PARTY_CATEGORY_HOME) and UnitIsGroupLeader("player")) then
        SendChatMessage("Used " .. spellName, "RAID_WARNING")
    elseif (c == "RAID" and IsInRaid(LE_PARTY_CATEGORY_HOME)) then
        SendChatMessage("Used " .. spellName, "RAID")
    elseif ((c == "RAID" or c == "PARTY") and IsInGroup(LE_PARTY_CATEGORY_HOME)) then
        SendChatMessage("Used " .. spellName, "PARTY")
    elseif ((c == "INSTANCE_CHAT" or c == "RAID" or c == "PARTY") and (IsInGroup(LE_PARTY_CATEGORY_INSTANCE) or IsInRaid(LE_PARTY_CATEGORY_INSTANCE))) then
        SendChatMessage("Used " .. spellName, "INSTANCE_CHAT")
    elseif (c == "YELL") then
        SendChatMessage("Used " .. spellName, "YELL")
    else
        SendChatMessage("Used " .. spellName, "SAY")
    end
end

function IM:UnitSpellcastStart(...)
    local unit = ...
    
    if (unit == "target" and IMDB.numInterrupters > 0) then
        local interruptImmune = select(9, UnitCastingInfo("target"))
        if (not interruptImmune and IMDB.targetWarn) then
            if (IMDB.soloMode or IM:PlayerAtPosition(1) == UnitName("player") .. "-" .. GetRealmName()) then
                InterruptManagerText:AddMessage("Interrupt now! (target)",0,0,1)
                PlaySoundFile("Sound\\Interface\\PVPFlagTakenMono.ogg")
            end
        end
    elseif (unit == "focus" and IMDB.numInterrupters > 0) then
        local interruptImmune = select(9, UnitCastingInfo("focus"))
        if (not interruptImmune and IMDB.focusWarn) then
            if (IMDB.soloMode or IM:PlayerAtPosition(1) == UnitName("player") .. "-" .. GetRealmName()) then
                InterruptManagerText:AddMessage("Interrupt now! (focus)",1,0,1)
                PlaySoundFile("Sound\\Interface\\PVPFlagTakenMono.ogg")
            end
        end
    end
end

function IM:CombatLogEvent(...)
    local event = select(2, ...)
    local sourceGUID = select(4, ...)
    local sourceName = select(5, ...)
    local spellId = select(12, ...)
    local spellName = select(13, ...)
    
    if (event == "SPELL_CAST_SUCCESS") then
        if (tContains(interruptSpells.spellId, spellId)) then
            -- If an interrupt spell was cast
            IM:InterruptUsed(sourceName, spellId)
            
            -- Announce my interrupt
            if (sourceGUID == UnitGUID("player") and IMDB.announce) then
                IM:AnnounceMyInterrupt(spellName)
            end
        end
    end
end

-- [Function to prevent system message spam] --
ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", function(self, event, msg)
    if(strfind(msg, "No player named ")) then
        --return true
    end
end)
local AntiSpam = {{},{}}
function IM:ChatMsgSystem(self, event, ...)
    local text = ...
    if (strfind(text, "No player named ")) then
        if (not tContains(AntiSpam[1],text)) then
            DEFAULT_CHAT_FRAME:AddMessage(text,1,1,0)
            tinsert(AntiSpam[1],text)
            tinsert(AntiSpam[2],GetTime()+1)
        else
            for k,v in pairs(AntiSpam[1]) do
                if (v == text) then
                    if (GetTime() >= AntiSpam[2][k]) then
                        AntiSpam[2][k] = GetTime()+1
                        DEFAULT_CHAT_FRAME:AddMessage(text,1,1,0)
                    end
                end
            end
        end
    end
end

SLASH_INTERRUPTMANAGER1 = '/im'
SLASH_INTERRUPTMANAGER2 = '/interruptmanager'
function SlashCmdList.INTERRUPTMANAGER(msg, editbox)
    if (string.find(msg, "lock")) then
        IM:OnFrameLock()
    else
        IM:OpenConfig()
        IM:InitializeNewFeatures()
    end
end